package com.example.demo.orders.model;

import java.util.Objects;

import com.example.demo.core.model.BaseEntity;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.users.model.UserEntity;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.Table;

// Сущность "Заказ" (Заказ, содержащий книги) 
@Entity
@Table(name = "orders")
public class OrderEntity extends BaseEntity {
    // Тип книги
    @ManyToOne
    @JoinColumn(name = "typeId", nullable = false)
    private TypeEntity type;

    // Заказчик (пользователь)
    @ManyToOne
    @JoinColumn(name = "userId", nullable = false)
    private UserEntity user;

    // Цена книги
    @Column(nullable = false)
    private Double price;

    // Количество книг
    @Column(nullable = false)
    private Integer count;

    // Конструктор по умолчанию
    public OrderEntity() {
    }

    // Конструктор с параметрами для создания объекта
    public OrderEntity(TypeEntity type, Double price, Integer count) {
        this.type = type;
        this.price = price;
        this.count = count;
    }

    // Получить тип книги
    public TypeEntity getType() {
        return type;
    }

    // Установить тип книги
    public void setType(TypeEntity type) {
        this.type = type;
    }

    // Получить заказчика (пользователя)
    public UserEntity getUser() {
        return user;
    }

    // Установить заказчика (пользователя)
    public void setUser(UserEntity user) {
        this.user = user;
        if (!user.getOrders().contains(this)) {
            user.getOrders().add(this);
        }
    }

    // Получить стоимость книги
    public Double getPrice() {
        return price;
    }

    // Установить стоимость книги
    public void setPrice(Double price) {
        this.price = price;
    }

    // Получить количество книг
    public Integer getCount() {
        return count;
    }

    // Установить количество книг
    public void setCount(Integer count) {
        this.count = count;
    }

    // Получить хэш-код объекта
    @Override
    public int hashCode() {
        return Objects.hash(id, type, user.getId(), price, count);
    }

    // Сравнить объекты
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final OrderEntity other = (OrderEntity) obj;
        return Objects.equals(other.getId(), id)
                && Objects.equals(other.getType(), type)
                && Objects.equals(other.getUser().getId(), user.getId())
                && Objects.equals(other.getPrice(), price)
                && Objects.equals(other.getCount(), count);
    }
}
