package com.example.demo.orders.repository;

import java.util.Optional;
import java.util.List;

import org.springframework.data.repository.CrudRepository;
import org.springframework.data.repository.PagingAndSortingRepository;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;

import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.model.OrderGrouped;

// Хранилище для сущности "Заказ" (Заказ, содержащий книги)
public interface OrderRepository extends CrudRepository<OrderEntity, Long>, PagingAndSortingRepository<OrderEntity, Long> {
    // Получить заказ по пользователю и идентификатору
    Optional<OrderEntity> findOneByUserIdAndId(Long userId, Long id);
    
    // Получить список заказов по пользователю
    List<OrderEntity> findByUserId(Long userId);

    // Получить список заказов по пользователю (с пагинацией)
    Page<OrderEntity> findByUserId(Long userId, Pageable pageable);

    // Получить список заказов по типу
    List<OrderEntity> findByUserIdAndTypeId(Long userId, Long typeId);

    // Получить список заказов по типу (с пагинацией)
    Page<OrderEntity> findByUserIdAndTypeId(Long userId, Long typeId, Pageable pageable);

    // Получить заказы, сгруппированные по типу
    // select
    // type.name,
    // coalesce(sum(order.price), 0),
    // coalesce(sum(order.count), 0)
    // from types as type
    // left join orders as order on type.id = order.type_id and order.user_id = ?
    // group by type.name order by type.id
    @Query("select "
            + "t as type, "
            + "coalesce(sum(o.price), 0) as totalPrice, "
            + "coalesce(sum(o.count), 0) as totalCount "
            + "from TypeEntity t left join OrderEntity o on o.type = t and o.user.id = ?1 "
            + "group by t order by t.id")
    List<OrderGrouped> getOrdersTotalByType(long userId);
}
